@@ -1,6 +1,6 @@
 # Software License Agreement (BSD License)
 #
-# Copyright (c) 2012, Willow Garage, Inc.
+# Copyright (c) 2013, Willow Garage, Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -50,18 +50,22 @@ from bloom.logging import warning
 
 from bloom.util import change_directory
 from bloom.util import check_output
-from bloom.util import code
 from bloom.util import execute_command
+from bloom.util import get_git_clone_state
 from bloom.util import pdb_hook
 import bloom.util
 
 
 class GitClone(object):
     def __init__(self, directory=None, track_all=True):
+        self.disabled = get_git_clone_state()
+        if self.disabled:
+            warning('Skipping transactional safety mechanism, be careful...')
+            return
         self.tmp_dir = None
         self.directory = directory if directory is not None else os.getcwd()
         if get_root(directory) is None:
-            raise RuntimeError("Provided directory, '" + str(directory) + \
+            raise RuntimeError("Provided directory, '" + str(directory) +
                                "', is not a git repository")
         self.track_all = track_all
         if self.track_all:
@@ -73,9 +77,17 @@ class GitClone(object):
         execute_command('git clone ' + self.repo_url + ' ' + self.clone_dir)
 
     def __del__(self):
+        if self.disabled:
+            return
         self.clean_up()
 
     def __enter__(self):
+        if self.disabled:
+            return
+        current_branch = get_current_branch()
+        if current_branch is None:
+            warning("Could not determine current branch, changing to the bloom branch")
+            execute_command('git checkout bloom')
         self.orig_cwd = os.getcwd()
         os.chdir(self.clone_dir)
         if self.track_all:
@@ -83,14 +95,23 @@ class GitClone(object):
         return os.getcwd()
 
     def __exit__(self, exc_type, exc_value, traceback):
+        if self.disabled:
+            return
         os.chdir(self.orig_cwd)
 
     def clean_up(self):
+        if self.disabled:
+            return
         if self.tmp_dir is not None and os.path.exists(self.tmp_dir):
             shutil.rmtree(self.tmp_dir)
             self.tmp_dir = None
 
     def commit(self):
+        if self.disabled:
+            return
+        current_branch = get_current_branch()
+        if current_branch is None:
+            error("Could not determine current branch.", exit=True)
         with inbranch(get_commit_hash(get_current_branch())):
             with change_directory(self.clone_dir):
                 new_branches = get_branches()
@@ -181,7 +202,7 @@ def show(reference, path, directory=None):
 
 def ensure_clean_working_env(force=False, git_status=True, directory=None):
     """
-    Returns 0 if the working environment is clean, otherwise 1.
+    Checks the environment to ensure it is clean, raises SystemExit otherwise.
 
     Clean is defined as:
         - In a git repository
@@ -192,39 +213,39 @@ def ensure_clean_working_env(force=False, git_status=True, directory=None):
     :param force: If True, overrides a few of the fail conditions
     :param directory: directory in which to run this command
 
-    :returns: 0 if the env is clean, otherwise 1
-
     :raises: subprocess.CalledProcessError if any git calls fail
+    :raises: SystemExit if any git calls fail
     """
     def ecwe_fail(code, show_git_status):
         if not bloom.util._quiet and show_git_status:
             print('\n++ git status:\n')
             os.system('git status')
-        return code
+        error(code, exit=True)
     # Is it a git repo
     if get_root(directory) is None:
-        error("Not is a valid git repository")
-        return code.NOT_A_GIT_REPOSITORY
+        error("Not is a valid git repository", exit=True)
     # Are we on a branch?
     current_branch = get_current_branch(directory)
     if current_branch is None:
-        msg = warning if force else error
-        msg("Could not determine current branch")
+        msg = "Could not determine current branch"
         if not force:
-            return ecwe_fail(code.NOT_ON_A_GIT_BRANCH, git_status)
+            return ecwe_fail(msg, git_status)
+        else:
+            warning(msg)
     # Are there local changes?
     if has_changes(directory):
-        msg = warning if force else error
-        msg("Current git working branch has local changes")
+        msg = "Current git working branch has local changes"
         if not force:
-            return ecwe_fail(code.GIT_HAS_LOCAL_CHANGES, git_status)
+            return ecwe_fail(msg, git_status)
+        else:
+            warning(msg)
     # Are there untracked files or directories?
     if has_untracked_files(directory):
-        msg = warning if force else error
-        msg("Current git working branch has untracked files/directories")
+        msg = "Current git working branch has untracked files/directories"
         if not force:
-            return ecwe_fail(code.GIT_HAS_UNTRACKED_FILES, git_status)
-    return 0
+            return ecwe_fail(msg, git_status)
+        else:
+            warning(msg)
 
 
 def checkout(reference, raise_exc=False, directory=None, show_git_status=True):
@@ -239,7 +260,7 @@ def checkout(reference, raise_exc=False, directory=None, show_git_status=True):
     def checkout_summarize(fail_msg, branch, directory):
         branch = '(no branch)' if branch is None else branch
         directory = os.getcwd() if directory is None else directory
-        error("Failed to checkout to '{0}'".format(str(reference)) + \
+        error("Failed to checkout to '{0}'".format(str(reference)) +
               " because the working directory {0}".format(str(fail_msg)))
         debug("  Working directory:   '{0}'".format(str(directory)))
         debug("  Working branch:      '{0}'".format(str(branch)))
@@ -249,7 +270,7 @@ def checkout(reference, raise_exc=False, directory=None, show_git_status=True):
         if not bloom.util._quiet and show_git_status:
             print('\n++ git status:\n')
             os.system('git status')
-        return 1
+        return False
     debug("Checking out to " + str(reference))
     if reference == get_current_branch(directory):
         debug("Requested checkout reference is the same as the current branch")
@@ -259,7 +280,7 @@ def checkout(reference, raise_exc=False, directory=None, show_git_status=True):
     if git_root is not None:
         changes = has_changes(directory)
         untracked = has_untracked_files(directory)
-        branch = get_current_branch(directory)
+        branch = get_current_branch(directory) or 'could not determine branch'
     else:
         fail_msg = "is not a git repository"
     if fail_msg == '' and changes:
@@ -279,7 +300,7 @@ def checkout(reference, raise_exc=False, directory=None, show_git_status=True):
     if fail_msg != '':
         return checkout_summarize(fail_msg, branch, directory)
     else:
-        return 0
+        return True
 
 
 class ContextDecorator(object):
@@ -320,7 +341,10 @@ class inbranch(ContextDecorator):
         checkout(self.branch, raise_exc=True, directory=self.directory)
 
     def __exit__(self, exc_type, exc_value, traceback):
-        checkout(self.current_branch, raise_exc=True, directory=self.directory)
+        if self.current_branch is not None:
+            checkout(self.current_branch, raise_exc=True, directory=self.directory)
+        else:
+            warning("Could not determine branch to return to.")
 
 
 def get_commit_hash(reference, directory=None):
@@ -369,6 +393,8 @@ def has_changes(directory=None):
     out = check_output('git status', shell=True, cwd=directory)
     if 'nothing to commit (working directory clean)' in out:
         return False
+    if 'nothing to commit, working directory clean' in out:
+        return False
     if 'nothing added to commit' in out:
         return False
     return True
@@ -387,6 +413,44 @@ def tag_exists(tag, directory=None):
     return tag in get_tags(directory)
 
 
+def create_tag(tag, directory=None):
+    """
+    Creates a given tag
+
+    :param tag: tag to create
+    :param directory: directory in which to preform this action
+
+    :raises: subprocess.CalledProcessError if any git calls fail
+    """
+    execute_command('git tag {0}'.format(tag), shell=True, cwd=directory)
+
+
+def delete_tag(tag, directory=None):
+    """
+    Deletes a given local tag.
+
+    :param tag: local tag to delete
+    :param directory: directory in which to preform this action
+
+    :raises: subprocess.CalledProcessError if any git calls fail
+    """
+    execute_command('git tag -d {0}'.format(tag), shell=True, cwd=directory)
+
+
+def delete_remote_tag(tag, remote='origin', directory=None):
+    """
+    Deletes a given remote tag.
+
+    :param tag: remote tag to delete
+    :param remote: git remote to delete tag from (defaults to 'origin')
+    :param directory: directory in which to preform this action
+
+    :raises: subprocess.CalledProcessError if any git calls fail
+    """
+    execute_command('git push {0} :{1}'.format(remote, tag), shell=True,
+                    cwd=directory)
+
+
 def get_tags(directory=None):
     """
     Returns a list of tags in the git repository.
