@@ -1,6 +1,6 @@
 # Software License Agreement (BSD License)
 #
-# Copyright (c) 2012, Willow Garage, Inc.
+# Copyright (c) 2013, Willow Garage, Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -38,17 +38,23 @@ from __future__ import print_function
 
 import argparse
 import os
+import shutil
 import sys
+import tempfile
 
 from subprocess import CalledProcessError
 from subprocess import PIPE
+from subprocess import STDOUT
 from subprocess import Popen
 
+from StringIO import StringIO
+
 from bloom.logging import ansi
 from bloom.logging import debug
 from bloom.logging import disable_ANSI_colors
 from bloom.logging import enable_debug
 from bloom.logging import error
+from bloom.logging import info
 from bloom.logging import warning
 
 try:
@@ -104,23 +110,60 @@ class change_directory(object):
             os.chdir(self.original_cwd)
 
 
-def get_package_data(branch_name, directory=None, quiet=False):
+class redirected_stdio(object):
+    def __enter__(self):
+        self.original_stdout = sys.stdout
+        self.original_stderr = sys.stderr
+        sys.stdout = out = StringIO()
+        sys.stderr = err = StringIO()
+        return out, err
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        sys.stdout = self.original_stdout
+        sys.stderr = self.original_stderr
+
+
+class temporary_directory(object):
+    def __init__(self, prefix=''):
+        self.prefix = prefix
+
+    def __enter__(self):
+        self.original_cwd = os.getcwd()
+        self.temp_path = tempfile.mkdtemp(prefix=self.prefix)
+        os.chdir(self.temp_path)
+        return self.temp_path
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if self.temp_path and os.path.exists(self.temp_path):
+            shutil.rmtree(self.temp_path)
+        if self.original_cwd and os.path.exists(self.original_cwd):
+            os.chdir(self.original_cwd)
+
+
+def get_package_data(branch_name, directory=None, quiet=True, fuerte=False):
     """
     Gets package data about the package(s) in the current branch.
 
     :param branch_name: name of the branch you are searching on (log use only)
     """
-    debug("Looking for packages in '{0}'... ".format(branch_name), end='')
-    ## Check for package.xml(s)
+    log = debug if quiet else info
     repo_dir = directory if directory else os.getcwd()
-    packages = find_packages(repo_dir)
+    stack_path = os.path.join(repo_dir, 'stack.xml')
+    if os.path.exists(stack_path) and not fuerte:
+            warning("stack.xml is present but going to be ignored because this is not a release for Fuerte.")
+    log("Looking for packages in '{0}' branch... ".format(branch_name), end='')
+    ## Check for package.xml(s)
+    if not fuerte:
+        packages = find_packages(repo_dir)
+    else:
+        packages = {}
     if type(packages) == dict and packages != {}:
         if len(packages) > 1:
-            debug("found " + str(len(packages)) + " packages.",
-                 use_prefix=False)
+            log("found " + str(len(packages)) + " packages.",
+                use_prefix=False)
         else:
-            debug("found '" + packages.values()[0].name + "'.",
-                 use_prefix=False)
+            log("found '" + packages.values()[0].name + "'.",
+                use_prefix=False)
         version = verify_equal_package_versions(packages.values())
         return [p.name for p in packages.values()], version, packages
     ## Check for stack.xml
@@ -129,53 +172,64 @@ def get_package_data(branch_name, directory=None, quiet=False):
         import rospkg
         has_rospkg = True
     except ImportError:
-        debug(ansi('redf') + "failed." + ansi('reset'), use_prefix=False)
+        log(ansi('redf') + "failed." + ansi('reset'), use_prefix=False)
         warning("rospkg was not detected, stack.xml discovery is disabled",
                 file=sys.stderr)
     if not has_rospkg:
         error("no package.xml(s) found, and no name specified with "
-              "'--package-name', aborting.", use_prefix=False)
-        return code.NO_PACKAGE_XML_FOUND
-    stack_path = os.path.join(repo_dir, 'stack.xml')
+              "'--package-name', aborting.", use_prefix=False, exit=True)
     if os.path.exists(stack_path):
-        debug("found stack.xml.", use_prefix=False)
+        log("found stack.xml.", use_prefix=False)
         stack = rospkg.stack.parse_stack_file(stack_path)
         return stack.name, stack.version, stack
     # Otherwise we have a problem
-    debug("failed.", use_prefix=False)
-    if not quiet:
-        error("no package.xml(s) or stack.xml found, and not name "
-              "specified with '--package-name', aborting.", use_prefix=False)
-    return code.NO_PACKAGE_XML_FOUND
+    log("failed.", use_prefix=False)
+    error("no package.xml(s) or stack.xml found, and no name "
+          "specified with '--package-name', aborting.",
+          use_prefix=False, exit=True)
 
 
 def add_global_arguments(parser):
-    group = parser.add_argument_group('global')
-    group.add_argument('-d', '--debug', help='enable debug messages',
-                       action='store_true', default=False)
-    group.add_argument('--pdb', help=argparse.SUPPRESS,
-                       action='store_true', default=False)
     from bloom import __version__
-    group.add_argument('--version', action='version', version=__version__,
-                       help="prints the bloom version")
-    group.add_argument('--no-color', action='store_true', default=False,
-                       dest='no_color', help=argparse.SUPPRESS)
+    group = parser.add_argument_group('global')
     add = group.add_argument
+    add('-d', '--debug', help='enable debug messages',
+        action='store_true', default=False)
+    add('--pdb', help=argparse.SUPPRESS,
+        action='store_true', default=False)
+    add('--version', action='version', version=__version__,
+        help="prints the bloom version")
+    add('--no-color', action='store_true', default=False,
+        dest='no_color', help=argparse.SUPPRESS)
     add('--quiet', help=argparse.SUPPRESS,
         default=False, action='store_true')
+    add('--unsafe', default=False, action='store_true',
+        help="Makes bloom faster, but if there is an error then you could run into trouble.")
     return parser
 
 _pdb = False
 _quiet = False
+_disable_git_clone = False
+
+
+def disable_git_clone(state=True):
+    global _disable_git_clone
+    _disable_git_clone = state
+
+
+def get_git_clone_state():
+    global _disable_git_clone
+    return _disable_git_clone
 
 
 def handle_global_arguments(args):
     global _pdb, _quiet
-    enable_debug(args.debug)
+    enable_debug(args.debug or 'DEBUG' in os.environ)
     _pdb = args.pdb
     _quiet = args.quiet
     if args.no_color:
         disable_ANSI_colors()
+    disable_git_clone(args.unsafe or 'BLOOM_UNSAFE' in os.environ)
 
 
 def print_exc(exc):
@@ -270,8 +324,9 @@ def extract_text(element):
 def segment_version(full_version):
     version_list = full_version.split('.')
     if len(version_list) != 3:
-        error('Invalid version element in the stack.xml, expected: ' \
-              '<major>.<minor>.<patch>')
+        warning('Invalid version element in the stack.xml, expected: '
+                '<major>.<minor>.<patch>')
+    if len(version_list) < 3:
         sys.exit(code.INVALID_VERSION)
     return version_list
 
@@ -281,24 +336,25 @@ def execute_command(cmd, shell=True, autofail=True, silent=True,
     """
     Executes a given command using vcstools' run_shell_command function.
     """
-    io_type = None
+    out_io = None
+    err_io = None
     result = 0
     if silent:
-        io_type = PIPE
+        out_io = PIPE
+        err_io = STDOUT
     debug(((cwd) if cwd else os.getcwd()) + ":$ " + str(cmd))
-    p = Popen(cmd, shell=True, cwd=cwd, stdout=io_type, stderr=io_type)
+    p = Popen(cmd, shell=True, cwd=cwd, stdout=out_io, stderr=err_io)
     out, err = p.communicate()
     result = p.returncode
     if result != 0:
         if not silent_error:
-            error("'execute_command' failed to call '{0}'".format(cmd) + \
+            error("'execute_command' failed to call '{0}'".format(cmd) +
                   " which had a return code ({0}):".format(result))
-            error("    stdout:\n" + ansi('reset') + str(out))
-            error("end stdout")
-            error("    stderr:\n" + ansi('reset') + str(err))
-            error("end stderr")
+            error("```")
+            print(out)
+            error("```")
         if autofail:
-            raise CalledProcessError(cmd=cmd, output=out, returncode=result)
+            raise CalledProcessError(result, cmd)
     if return_io:
         return result, out, err
     else:
