@@ -11,12 +11,14 @@ from ..utils.common import change_directory
 from ..utils.common import user
 
 
-def create_release_repo(upstream_url, upstream_type, upstream_branch=''):
+def create_release_repo(upstream_url, upstream_type, upstream_branch='',
+    rosdistro='groovy'):
     user('mkdir foo_release')
     with change_directory('foo_release'):
         user('git init .')
-        cmd = 'git-bloom-config'
-        with bloom_answer('y'):
-            user(' '.join([cmd, upstream_url, upstream_type, upstream_branch]))
+        answers = ['y', 'foo', upstream_url, upstream_type,
+            '', '', upstream_branch, rosdistro]
+        with bloom_answer(answers):
+            user('git-bloom-config new ' + str(rosdistro))
         url = 'file://' + os.getcwd()
     return url
